Documentation
¶
Overview ¶
Package agentloop implements an iterative agent loop that can call tools.
Index ¶
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
This section is empty.
Types ¶
type Factory ¶
type Factory struct {
// contains filtered or unexported fields
}
Factory builds agent iteration activities.
func NewFactory ¶
func NewFactory(invokeLLMFactory executor.ActivityFactory[*draftcontent.Input, *draftcontent.Output]) (*Factory, error)
NewFactory creates a new agent iteration factory.
type Input ¶
type Input struct {
ToolRegistry *tools.Registry
ToolDescriptionOverrides map[string]string
Preset *preset.ResolvedPreset
PriorSummaries *[]string
Goal *string
SystemPrompt *string
MaxIterations *int
StepName string
AllowedTools []string
}
Input defines the agent iteration input parameters.
type Output ¶
type Output struct {
// Content is the full user-facing output for the step, including narration
// emitted in tool-call iterations plus the final answer.
Content string
// FinalMessage is the message emitted in the first iteration with no tool calls.
// This is typically what subsequent steps should use as the prior step output.
FinalMessage string
Summary string
}
Output defines the agent iteration output.
Click to show internal directories.
Click to hide internal directories.